%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Developed by:			Torbjrn Wigren
%                       Division of Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       P O Box 337
%                       SE-751 05 Uppsala
%                       SWEDEN
%
% Version:					1.0, PC MATLAB
%                       2.0, MATLAB 5
%
% Date:		        		rev. 1.0, Nov 10 1991
%                       rev. 2.0, March 28 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       DESCRIPTION
%
% This M-file implements a RPE-algorithm that identifies a static
% nonlinearity. The nonlinearity is approximated with a piecevise quadratic 
% curve. The algorithm is referred to as algorithm 10 in Wigren 1990.
%
% Variables:         u = input signal
%                    yn = output signal 
%                    n = number of samples 	
%                    algnr = number of algorithm (=10)
%                    fymod = output from the nonlinearity
%                    ynmod = output from the nonlinear model (=fymod)
%		               thetaoe = parameter vector containing f(yi)
%		               theta0 = initial value of parameter vector
%		               thetapro = dummy vector
%		               lambda = gain sequence
%		               perr = nonlinear prediction error
%		               S = scalar in Riccattieequation
%		               L = gain vector
%		               P = P-matrix in Riccattieequation
%		               lambda0 = rate of gain sequence (0.99 in L&S)
%		               kpl = kplus in the notes
%		               kmi = kminus in the notes
%                    k0 = slope used in initialization of algorithm
%                    thetafy0_0 = bias parameter used in initialization
%                    grid = vector containing grid points
%                    extgrid = grid including middle points
%		               thetanon_0 = starting value of nonlinear parameter vector
%		               psi = total gradient vector
%		               Intervals = all interval where u < upper interval limit
%		               i = upper limit index in interval to which u belongs
%		               ulow = lower limit of interval to which u belongs
%		               umid = middle point in interval to which u belongs
%                    uhigh = upper limit of interval to which u belongs
%                    fulow = parameter value in ulow
%		               fumid = parameter value in umid
%                    fuhigh = parameter value in uhigh
%		               psinon = nonlinear part of the gradient
%		               deltai = width of approximation interval on positive side
%		               deltamii = width of approximation interval on negative ..
%                    dummy = dummy variable
%		               psifyo = df/df(y0)
%                    lambda_0 = initial value of gain sequence
%                    P_0 = initial value of P-matrix
%                    size_P_0 = dimension of P_0
%                    thetanon0 = dummy variable (=thetanon_0)
%                    size_thetanon_0 = dimension of thetanon_0
%                    inside = flag determining if inside all intervals
%
% The following variables must be present in the working area before the
% program is run (i.e user initializations or data):
%
%                    n = number of data samples
%                    u = vector containing input signal samples
%                    yn = vector containng output signal samples
%                    k0 = slope used in initialization of algorithm
%                    thetafy0_0 = bias parameter used in initialization
%                    kmi = kminus in the notes
%                    kpl = kplus in the notes
%                    grid = vector of grid points
%                    lambda0 = rate of gain sequence
%                    lambda_0 = initial value of gain sequence
%                    P_0 = initial value of P-matrix
%                    thetanon_0 = initial value of nonlinearity (column vec)
%
% Note that by specifying P_0 = [], the default value of a unit matrix
% of appropriate dimension is applied. If thetanon_0 is choosen  as []
% then a straight line initial value with slope k0 and bias thetafy0_0
% is used. Since the mid-points of the approximation intervals are
% specified by the algorithm they shall not be included in grid.
% thetanon_0 must however be specified completely.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% User initialization.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
lambda=lambda_0;
size_P_0=size(P_0);
if size_P_0(1,1)==0
  P_0=1*eye(2*kpl+2*kmi+1);
  P=P_0;
end;
if size_P_0(1,1)>0
  P=P_0;
end;
extgrid=[grid;zeros(1,kmi+kpl+1)];
extgrid=extgrid(:)';
extgrid=extgrid(1:2*kmi+2*kpl+1);
for i=1:kmi+kpl
  extgrid(2*i)=0.5*(extgrid(2*i+1)+extgrid(2*i-1));
end;
size_thetanon_0=size(thetanon_0);
if size_thetanon_0(1,1)==0
  thetanon_0=k0*extgrid(1:2*kmi)';
  thetanon_0=[thetanon_0;k0*extgrid(2*kmi+2:2*kmi+2*kpl+1)']+thetafy0_0;
  thetanon0=thetanon_0;
end;
if size_thetanon_0(1,1)>0
  thetanon0=thetanon_0;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Other standard initializations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
algnr=10;
theta0=[k0*grid(2*kmi+1);thetanon0];
psi=zeros(2*kmi+2*kpl+1,1);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Also the time variables need initial values.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
perr=zeros(n,1);
thetaoe=zeros(n,2*kmi+2*kpl+1);
ynmod=zeros(n,1);
fymod=0;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now the recursive algorithm starts. t is the time.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
t=1;
inside=1;
while t < n
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute prediction errors using prediction from previous iteration and
% save the result for later use.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  perr(t)=yn(t)-ynmod(t);
  if inside==0
    perr(t)=0;
  end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Then a new lambda-value is computed.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  lambda=lambda0*lambda+1-lambda0;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The next step is to iterate the Riccattieequation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  S=psi'*P*psi+lambda;
  P=(P-P*psi*psi'*P/S)/lambda;
  L=P*psi;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next the thetaoe-vector is uppdated and saved for later use. To get the same
% time indices for u, yn, ynmod, perr and thetaoe we distinguish between the
% cases t=1 and t>1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  if t==1
    thetapro=theta0+L*perr(t);
  end;
  if t>=2
    thetapro=thetaoe(t-1,:)'+L*perr(t);
  end;
  thetaoe(t,:)=thetapro';
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now the output of the nonlinearity is computed. Also
% df/dthetanon is computed. There are four cases to consider depending
% on which approximation interval u(t+1) belongs to.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  psify0=0;
  psinon=zeros(2*kmi+2*kpl,1);
  inside=0;
  if u(t+1) > grid(1);
    if u(t+1) < grid(kmi+kpl+1);
      inside=1;
      Intervals=find(grid > u(t+1));
      i=Intervals(1);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
      if i<=kmi
        ulow=extgrid(2*i-3);
        umid=extgrid(2*i-2);
        uhigh=extgrid(2*i-1);
        fulow=thetaoe(t,1+2*i-3);
        fumid=thetaoe(t,1+2*i-2);
        fuhigh=thetaoe(t,1+2*i-1);
        deltamii=uhigh-ulow;
        dummy=2*(fuhigh-2*fumid+fulow)*(u(t+1)-umid)*(u(t+1)-umid);
        fymod=fumid+(fuhigh-fulow)/deltamii*(u(t+1)-umid);
        fymod=fymod+dummy/(deltamii*deltamii);
        dummy=(u(t+1)-umid)*(u(t+1)-umid)/(deltamii*deltamii);
	psinon(2*i-3)=2*dummy-(u(t+1)-umid)/deltamii;
        psinon(2*i-2)=1-4*dummy;
        psinon(2*i-1)=(u(t+1)-umid)/deltamii+2*dummy;
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 2.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
      if i==kmi+1
        ulow=extgrid(2*i-3);
        umid=extgrid(2*i-2);
        uhigh=extgrid(2*i-1);
        fulow=thetaoe(t,1+2*i-3);
        fumid=thetaoe(t,1+2*i-2);
        fuhigh=thetaoe(t,1);
        deltamii=uhigh-ulow;
        dummy=2*(fuhigh-2*fumid+fulow)*(u(t+1)-umid)*(u(t+1)-umid);
        fymod=fumid+(fuhigh-fulow)/deltamii*(u(t+1)-umid);
        fymod=fymod+dummy/(deltamii*deltamii);
        dummy=(u(t+1)-umid)*(u(t+1)-umid)/(deltamii*deltamii);
	psinon(2*i-3)=2*dummy-(u(t+1)-umid)/deltamii;
        psinon(2*i-2)=1-4*dummy;
        psify0=(u(t+1)-umid)/deltamii+2*dummy;
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 3.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
      if i==kmi+2
        ulow=extgrid(2*i-3);
        umid=extgrid(2*i-2);
        uhigh=extgrid(2*i-1);
        fulow=thetaoe(t,1);
        fumid=thetaoe(t,1+2*i-3);
        fuhigh=thetaoe(t,1+2*i-2);
        deltai=uhigh-ulow;
        dummy=2*(fuhigh-2*fumid+fulow)*(u(t+1)-umid)*(u(t+1)-umid);
        fymod=fumid+(fuhigh-fulow)/deltai*(u(t+1)-umid);
        fymod=fymod+dummy/(deltai*deltai);
        dummy=(u(t+1)-umid)*(u(t+1)-umid)/(deltai*deltai);
	psinon(2*i-3)=1-4*dummy;
        psinon(2*i-2)=(u(t+1)-umid)/deltai+2*dummy;
        psify0=2*dummy-(u(t+1)-umid)/deltai;
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 4.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
      if i>kmi+2
        ulow=extgrid(2*i-3);
        umid=extgrid(2*i-2);
        uhigh=extgrid(2*i-1);
	fulow=thetaoe(t,1+2*i-4);
        fumid=thetaoe(t,1+2*i-3);
	fuhigh=thetaoe(t,1+2*i-2);
        deltai=uhigh-ulow;
        dummy=2*(fuhigh-2*fumid+fulow)*(u(t+1)-umid)*(u(t+1)-umid);
        fymod=fumid+(fuhigh-fulow)/deltai*(u(t+1)-umid);
        fymod=fymod+dummy/(deltai*deltai);
        dummy=(u(t+1)-umid)*(u(t+1)-umid)/(deltai*deltai);
	psinon(2*i-4)=2*dummy-(u(t+1)-umid)/deltai;
	psinon(2*i-3)=1-4*dummy;
        psinon(2*i-2)=2*dummy+(u(t+1)-umid)/deltai;
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% End cases.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
    end;
  end;
  ynmod(t+1)=fymod;
  psi=[psify0;psinon];
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% That completes the loop.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  t=t+1;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Variables that are not used by subsequent programs are cleared.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
clear fymod;
clear thetapro;
clear theta0;
clear lambda;
clear t;
clear S;
clear L;
clear psi;
clear Intervals;
clear i;
clear ulow;
clear umid;
clear uhigh;
clear fulow;
clear fumid;
clear fuhigh;
clear psinon;
clear psify0;
clear size_P_0;
clear thetanon0;
clear size_thetanon_0;
clear inside;
clear deltai;
pack;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write parameter vector.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
theta=thetaoe(n-1,:);
theta
