%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Developed by:			Torbjrn Wigren
%                       Division of Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       P O Box 337
%                       SE-751 05 Uppsala
%                       SWEDEN
%
% Version:					1.0, PC MATLAB
%                       2.0, MATLAB 5
%                       3.0, Added FIR
%
% Date:		        		rev. 1.0, Nov 10 1991
%                       rev. 2.0, March 28 2007
%                       rev. 3.0, April 12 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       DESCRIPTION
%
% This M-file implements the algorithm that identifies the linear part
% of a nonlinear Wiener type model when the output nonlinearity is known
% a priori. The gradient is approximated with a filtered regression vector
% containing delayed inputs and model outputs, multiplied with a scaling factor
% beeing equal to df/dy(y), where f is the static nonlinearity.
%
% The static nonlinearity shall be implemented in the function eval_fn
% together with the corresponding derivative.
%
% Functions called:  eval_fn
%
% Variables:         u = input signal
%                    yn = output signal
%                    n = umber of samples 
%                    algnr = number of algorithm (=4)
%                    ymod = output from the linear part of the model	
%                    fymod = output from the nonlinear part of the model
%                    ynmod = output from the total nonlinear model (=fymod)
%		               dfdy = derivative of static nonlinearity wrt y
%                    a = identified A-polynomial
%                    b = identified B-polynomial
%		               d = prefilter for regressor filtering
%		               na = order of A-polynomial
%		               nb = order of B-polynomial
%		               nd = order of D-polynomial (normally = na)
%		               phi = regression vector
%		               phifilt = filtered regression vector
%		               phifilty = filtered regression vector (y-part)
%                    phifiltu = filtered regression vector (u-part)
%                    ufilt = filtered version of u
%                    yfilt = filtered version of y
%		               thetaoe = parameter vector containing a and b
%		               theta0 = initial value of parameter vector
%		               thetapro = parameter vector to be projected into Ds
%		               thetad = parameters of prefilter
%		               lambda = gain sequence
%		               perr = nonlinear prediction error
%		               L = gain vector
%		               p = p-scalar used in the stochastic gradient algoritm
%                    a0 = initial value of a-polynomial
%		               b0 = initial value of b-polynomial
%		               lambda0 = rate of gain sequence
%		               zerovec = vector used to update phi
%		               delayy = matrix used to update phi
%		               delayu = matrix used to update phi
%		               delay = matrix composed of delayy and delayu
% 		               theta = estimated parameter vector
%                    nf = order of estimated denominator polynomial (=nb)
%                    F_0 = initial value of denominator polynomial
%                    B_0 = initial value of numerator polynomial
%                    lambda_0 = initial value of gain sequence
%                    p_0 = initial value of (scalar) p
%
% The following variables must be present in the working area before
% the program is run (i.e. user initializations or data):
%                    n = number of data samples
%                    u = vector containing input signal samples
%                    yn = vector containing output signal samples
%                    F_0 = initial value of denominator polynomial
%                    B_0 = initial value of numerator polynomial
%                    lambda0 = rate of gain sequence
%                    lambda_0 = initial value of gain sequence
%                    p_0 = initial value of (scalar) p
%
% Note that the d-polynomial is automatically choosen equal to F_0. Also
% note that the orders of the numerator and the denominator are equal.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% User initialization.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
nf=length(F_0)-1;
na=nf;
nb=length(B_0)-1;% Added handling of FIR case
if na>0 % Added handling of FIR case
   a0=F_0;
end
b0=B_0;
lambda=lambda_0;
p=p_0;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Other standard initializations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
algnr=4;
nd=na;
if na>0 % Added handling of FIR case
   a=a0;
end
b=b0;
if na>0  % Added handling of FIR case
   theta0=[a0(2:na+1)';b0(2:nb+1)'];
else
   theta0=[b0(2:nb+1)'];
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The d-polynomial is to be an a-priori guess of a0dis so a rather obvious
% choice is
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if na>0 % Added handling of FIR case
	d=a0;
   thetad=d(2:nd+1)';
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% It remains to give initial values to phi. Since phi consists of inputs and
% delayed linear model outputs the result is
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
phi=zeros(na+nb,1);
phifilt=phi;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Also the other time variables need initial values.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
perr=zeros(n,1);
thetaoe=zeros(n,na+nb);
ufilt=zeros(n,1);
yfilt=zeros(n,1);
ymod=zeros(n,1);
ynmod=zeros(n,1);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% To efficiently update phi a zerovector and a delay matrix has to be created.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
zerovec=zeros(na+nb,1);
if na>0 % Added handling of FIR case
	if na>1
  		delayy=[[zeros(1,na-1);eye(na-1)]';zeros(1,na)]';
	else
  		delayy=0;
	end
	if nb>1
  		delayu=[[zeros(1,nb-1);eye(nb-1)]';zeros(1,na)]';
	else
  		delayu=0;
	end
   delay=[[delayy';zeros(nb,na)]';[zeros(na,nb);delayu']'];
else % Added handling of FIR case
	if nb>1
  		delayu=[[zeros(1,nb-1);eye(nb-1)]';zeros(1,nb)]';
	else
  		delayu=0;
	end
   delay=[[delayu']'];
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now the recursive algorithm starts. t is the time.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
t=1;
while t < n
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute prediction errors using prediction from previous iteration and
% save the result for later use in e.g. plotting routines.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  perr(t)=yn(t)-ynmod(t);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Then a new lambda-value is computed.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  lambda=lambda0*lambda+1-lambda0;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The next step is to iterate the gradient-direction.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  p=p/(lambda+p*phifilt'*phifilt);
  L=p*phifilt;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next the thetaoe-vector is uppdated and saved for later use. To get the same
% time indices for u, yn, ynmod, perr, d and thetaoe we distinguish between the
% cases t=1 and t>1. Projection into the stability set is used.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  if t==1
    thetapro=theta0+L*perr(t);
  end;
  if t>=2
    thetapro=thetaoe(t-1,:)'+L*perr(t);
  end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stability check.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  if na>0 % Added handling of FIR case
	  	a=[1;thetapro(1:na)]';
     	if all(abs(roots(a))<1)
    		 thetaoe(t,:)=thetapro';
     	end;
     	if any(abs(roots(a))>=1)
       	 thetaoe(t,:)=thetaoe(t-1,:);
      end;
   else % Added handling of FIR case 
      thetaoe(t,:)=thetapro';
   end;   
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The model output at time t+1 is computed using the parametervector at
% time t. This means that phi has to be updated.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  deltaphi=zerovec;
  deltaphi(1)=-ymod(t);
  deltaphi(na+1)=u(t);
  phi=delay*phi+deltaphi;
  ymod(t+1)=thetaoe(t,:)*phi;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Here the static nonlinearity and the corresponding derivative are
% evaluated by calling eval_fn.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  [fymod,dfdy]=eval_fn(ymod(t+1));
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Update model output.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  ynmod(t+1)=fymod;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The last thing to do in the loop is to compute the gradient in this case
% phifilt. This is obtained by introducing the two auxiliary varibles yfilt
% and ufilt.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
	if na>0 % Added handling of FIR case
 		 if t<nd
   	    yfilt(t+1)=-thetad'*[yfilt(t:-1:1);zeros(nd-t,1)]+ymod(t+1);
    		 ufilt(t+1)=-thetad'*[ufilt(t:-1:1);zeros(nd-t,1)]+u(t+1);
  	    else
    		 yfilt(t+1)=-thetad'*yfilt(t:-1:t-nd+1)+ymod(t+1);
    		 ufilt(t+1)=-thetad'*ufilt(t:-1:t-nd+1)+u(t+1);
       end;
    else % Added handling of FIR case
       yfilt(t+1)=ymod(t+1);
       ufilt(t+1)=u(t+1);
    end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% After creating yfilt and ufilt it is a simple matter to create the filtered
% regression vector at time t+1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  	if na>0 % Added handling of FIR case 		  
  	  	if t<na
     		phifilty=[yfilt(t:-1:1);zeros(na-t,1)];
  		else
   		phifilty=yfilt(t:-1:t-na+1);
  		end;
	   if t<nb
    		phifiltu=[ufilt(t:-1:1);zeros(nb-t,1)];
  		else
    		phifiltu=ufilt(t:-1:t-nb+1);
  		end;
        phifilt=[-1*phifilty;phifiltu];
   else % Added handling of FIR case
	   if t<nb
    		phifiltu=[ufilt(t:-1:1);zeros(nb-t,1)];
  		else
    		phifiltu=ufilt(t:-1:t-nb+1);
  		end;
        phifilt=[phifiltu];
   end        
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% To complete the gradient calculation phifilt is multiplied with df/dy(ymod).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  phifilt=dfdy*phifilt;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% That completes the loop.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  t=t+1;
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Variables that are not used by subsequent programs are cleared.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
clear t;
clear ymod;
clear ufilt;
clear yfilt;
clear theta0;
clear fymod;
clear a;
clear b;
clear d;
clear nd;
clear phi;
clear deltaphi;
clear phifilt;
clear phifiltu;
clear phifilty;
clear thetapro;
clear thetad;
clear L;
clear zerovec;
clear delayu;
clear delayy;
clear delay;
clear dfdy;
clear lambda;
pack;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write parameter vector.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
theta=thetaoe(n-1,:);
theta
