%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Developed by:			Torbjrn Wigren
%                       Division of Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       P O Box 337
%                       SE-751 05 Uppsala
%                       SWEDEN
%
% Version:					1.0, PC MATLAB
%                       2.0, MATLAB 5
%                       3.0, Added FIR
%
% Date:		        		rev. 1.0, Nov 10 1991
%                       rev. 2.0, March 28 2007
%                       rev. 3.0, April 12 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       DESCRIPTION
%
% This M-file implements a RPE-algorithm that identifies the linear and
% nonlinear parts of a nonlinear Wiener type model. The nonlinearity is
% approximated with a piecevise linear curve. The method is referred to 
% as algorithm 7 in Wigren 1990.
%
% Variables:         u = input signal 
%                    yn = output signal 
%                    n = number of samples 
%                    algnr = number of algorithm (=7)
%                    ymod = output from the linear part of the model	
%                    fymod = output from the nonlinear part of the model
%                    ynmod = output from the total nonlinear model (=fymod)
%		               dfdy = derivative of static nonlinearity wrt y
%                    a = identified A-polynomial
%                    b = identified B-polynomial
%		               d = prefilter for regressor filtering
%		               na = order of A-polynomial
%		               nb = order of B-polynomial
%		               nd = order of D-polynomial (normally = na)
%		               phi = regression vector for the linear part
%		               phifilt = filtered regression vector for the linear part
%		               phifilty = filtered regression vector (y-part)
%                    phifiltu = filtered regression vector (u-part)
%                    ufilt = filtered version of u
%                    yfilt = filtered version of y
%		               thetaoe = parameter vector containing a,b and f(yi)
%		               theta0 = initial value of parameter vector
%		               thetapro = parameter vector to be projected into Ds
%		               thetad = parameters of prefilter
%		               lambda = gain sequence
%		               perr = nonlinear prediction error
%		               S = scalar in Riccattieequation
%		               L = gain vector
%		               P = P-matrix in Riccattieequation
%                    a0 = initial value of a-polynomial
%		               b0 = initial value of b-polynomial
%		               lambda0 = rate of gain sequence (0.99 in L&S)
%		               zerovec = vector used to update phi
%		               delayy = matrix used to update phi
%		               delayu = matrix used to update phi
%		               delay = matrix composed of delayy and delayu
%		               kpl = kplus in the notes
%		               kmi = kminus in the notes
%                    k0 = known slope
%                    grid = vector containing grid points
%		               thetalin0 = starting value of linear parameter vector
%		               thetanon_0 = starting value of nonlinear parameter vector
%		               psi = total gradient vector
%		               Intervals = all interval where ymod < upper interval limit
%		               i = upper limit index in interval to which ymod belongs
%		               ylow = lower limit of interval to which ymod belongs
%                    yhigh = upper limit of interval to which ymod belongs
%                    fylow = parameter value in ylow
%                    fyhigh = parameter value in yhigh
%		               psinon = nonlinear part of the gradient
%		               thetafy0_0 = initial value of fy0
%		               psify0 = df/dfy0
%                    thetalinear = linear part of estimated parameter vector
%                    thetanonlinear = nonlinear part of estimated parameters
%                    nf = order of estimated denominator polynomial (=nb)
% 		               F_0 = initial value of denominator polynomial
%                    B_0 = initial value of numerator polynomial
%                    lambda_0 = initial value of gain sequence
%                    P_0 = initial value of P-matrix
%                    size_P_0 = dimension of P_0
%                    thetanon0 = dummy variable (=thetanon_0)
%                    size_thetanon_0 = dimension of thetanon_0
%                    inside = flag determining if inside all intervals
%
% The following variables must be present in the working area before the
% program is run (i.e. user initializations or data):
%
%                    n = number of data samples
%                    u = vector containing input signal samples
%                    yn = vector containing output signal samples
%                    F_0 = initial value of denominator polynomial
%                    B_0 = initial value of numerator polynomial
%                    k0 = fixed slope in interval I_0
%                    kmi = kminus in the notes
%                    kpl = kplus in the notes
%                    grid = vector of grid points
%                    thetafy0_0 = initial value of bias parameter in I_0
%                    lambda0 = rate of gain sequence
%                    lambda_0 = initial value of gain sequence
%                    P_0 = initial value of P_matrix
%                    thetanon_0 = initial value of nonlinearity (column vec)
%
% Note that nf=nb is assumed. Also note that by specifying P_0=[], the default
% value of a unit matrix of appropriate dimension is applied. If thetanon_0
% is choosen as [] then a straight line initial value with slope k0 is used
% as default.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% User initialization.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
nf=length(F_0)-1;
na=nf;
nb=length(B_0)-1;% Added handling of FIR case
if na>0 % Added handling of FIR case
   a0=F_0;
end
b0=B_0;
lambda=lambda_0;
size_P_0=size(P_0);
if size_P_0(1,1)==0 
  P_0=1*eye(na+nb+1+kpl+kmi);
  P=P_0;
end;
if size_P_0(1,1)>0
  P=P_0;
end;
size_thetanon_0=size(thetanon_0);
if size_thetanon_0(1,1)==0
  thetanon_0=[k0*grid(1:kmi)';k0*grid((kmi+3):(kmi+kpl+2))']+thetafy0_0;
  thetanon0=thetanon_0;
end;
if size_thetanon_0(1,1)>0
  thetanon0=thetanon_0;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Other standard initializations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
algnr=7;
nd=na;
if na>0 % Added handling of FIR case
   a=a0;
end
b=b0;
if na>0  % Added handling of FIR case
   thetalin0=[a0(2:na+1)';b0(2:nb+1)'];
else
   thetalin0=[b0(2:nb+1)'];
end
theta0=[thetalin0;thetafy0_0;thetanon0];
dfdy=k0;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The d-polynomial is to be an a-priori guess of a0dis so a rather obvious
% choice is
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if na>0 % Added handling of FIR case
	d=a0;
   thetad=d(2:nd+1)';
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% It remains to give initial values to phi. Since phi consists of inputs and
% delayed linear model outputs the result is
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
phi=zeros(na+nb,1);
phifilt=phi;
psi=zeros(na+nb+1+kmi+kpl,1);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Also the time variables need initial values.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
perr=zeros(n,1);
thetaoe=zeros(n,na+nb+1+kmi+kpl);
ufilt=zeros(n,1);
yfilt=zeros(n,1);
ymod=zeros(n,1);
ynmod=zeros(n,1);
fymod=0;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% To efficiently update phi a zerovector and a delay matrix has to be created.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
zerovec=zeros(na+nb,1);
if na>0 % Added handling of FIR case
	if na>1
  		delayy=[[zeros(1,na-1);eye(na-1)]';zeros(1,na)]';
	else
  		delayy=0;
	end
	if nb>1
  		delayu=[[zeros(1,nb-1);eye(nb-1)]';zeros(1,na)]';
	else
  		delayu=0;
	end
   delay=[[delayy';zeros(nb,na)]';[zeros(na,nb);delayu']'];
else % Added handling of FIR case
	if nb>1
  		delayu=[[zeros(1,nb-1);eye(nb-1)]';zeros(1,nb)]';
	else
  		delayu=0;
	end
   delay=[[delayu']'];
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now the recursive algorithm starts. t is the time.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
t=1;
inside=1;
while t < n
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute prediction errors using prediction from previous iteration and
% save the result for later use in e.g. plotting routines.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  perr(t)=yn(t)-ynmod(t);
  if inside==0
    perr(t)=0;
  end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Then a new lambda-value is computed.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  lambda=lambda0*lambda+1-lambda0;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  The next step is to iterate the Riccattieequation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  S=psi'*P*psi+lambda;
  P=(P-P*psi*psi'*P/S)/lambda;
  L=P*psi;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next the thetaoe-vector is uppdated and saved for later use. To get the same
% time indices for u, yn, ynmod, perr, d and thetaoe we distinguish between the
% cases t=1 and t>1. Projection into the stability set is used. Also linear
% outputs outside the approximation interval are disregarded. This does not
% change the result since psinon is anyway zero as explained in Wigren 1990.
% Some calculations are avoided though.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  if t==1
    thetapro=theta0+L*perr(t);
  end;
  if t>=2
    thetapro=thetaoe(t-1,:)'+L*perr(t);
  end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stability check and amplitude check.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
 if t==1
    thetaoe(t,:)=thetapro';
 end
 if t>1
    thetaoe(t,:)=thetaoe(t-1,:);
 end
 if na>0 % Added handling of FIR case
	  	a=[1;thetapro(1:na)]';
     	if all(abs(roots(a))<1)
         if ymod(t)>grid(1)
            if ymod(t)<grid(kmi+kpl+2)
            	thetaoe(t,:)=thetapro';
            end
         end
     	end;
   else % Added handling of FIR case 
      if ymod(t)>grid(1)
         if ymod(t)<grid(kmi+kpl+2)
         	thetaoe(t,:)=thetapro';
         end
      end
   end;   
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The linear model output at time t+1 is computed using the parametervector at
% time t. This means that phi has to be updated.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  deltaphi=zerovec;
  deltaphi(1)=-ymod(t);
  deltaphi(na+1)=u(t);
  phi=delay*phi+deltaphi;
  ymod(t+1)=thetaoe(t,1:na+nb)*phi;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now the output of the nonlinear part is computed. Also df/dy and
% df/dthetanon is computed. There are five cases to consider depending
% on which approximation interval ymod(t+1) belongs to. First the gradient
% components related to the nonlinear part of the parameter vector are
% filled with zeros.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  psify0=0;
  psinon=zeros(kmi+kpl,1);
  inside=0;
  if ymod(t+1) > grid(1);
    if ymod(t+1) < grid(kmi+kpl+2);
      inside=1;
      Intervals=find(grid > ymod(t+1));
      i=Intervals(1);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
      if i<=kmi
        ylow=grid(i-1);
        yhigh=grid(i);
        fylow=thetaoe(t,na+nb+1+i-1);
        fyhigh=thetaoe(t,na+nb+1+i);
        fymod=(fylow*yhigh-fyhigh*ylow+(fyhigh-fylow)*ymod(t+1))/(yhigh-ylow);
        dfdy=(fyhigh-fylow)/(yhigh-ylow);
        psinon(i-1)=(yhigh-ymod(t+1))/(yhigh-ylow);
        psinon(i)=1-psinon(i-1);
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 2.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
      if i==kmi+1
        ylow=grid(i-1);
        yhigh=grid(i);
        fylow=thetaoe(t,na+nb+1+i-1);
        fyhigh=k0*yhigh+thetaoe(t,na+nb+1);
        fymod=(fylow*yhigh-fyhigh*ylow+(fyhigh-fylow)*ymod(t+1))/(yhigh-ylow);
        dfdy=(fyhigh-fylow)/(yhigh-ylow);
        psinon(i-1)=(yhigh-ymod(t+1))/(yhigh-ylow);
        psify0=(ymod(t+1)-ylow)/(yhigh-ylow);
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 3.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
      if i==kmi+2
        fymod=k0*ymod(t+1)+thetaoe(t,na+nb+1);
        dfdy=k0;
        psify0=1;
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 4.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
      if i==kmi+3
        ylow=grid(i-1);
        yhigh=grid(i);
        fylow=k0*ylow+thetaoe(t,na+nb+1);
        fyhigh=thetaoe(t,na+nb+1+i-2);
        fymod=(fylow*yhigh-fyhigh*ylow+(fyhigh-fylow)*ymod(t+1))/(yhigh-ylow);
        dfdy=(fyhigh-fylow)/(yhigh-ylow);
        psinon(i-2)=(ymod(t+1)-ylow)/(yhigh-ylow);
        psify0=(yhigh-ymod(t+1))/(yhigh-ylow);
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% case 5.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
     if i>kmi+3
        ylow=grid(i-1);
        yhigh=grid(i);
        fylow=thetaoe(t,na+nb+1+i-3);
        fyhigh=thetaoe(t,na+nb+1+i-2);
        fymod=(fylow*yhigh-fyhigh*ylow+(fyhigh-fylow)*ymod(t+1))/(yhigh-ylow);
        dfdy=(fyhigh-fylow)/(yhigh-ylow);
        psinon(i-3)=(yhigh-ymod(t+1))/(yhigh-ylow);
        psinon(i-2)=1-psinon(i-3);
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Cases completed.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
    end;
  end;
  ynmod(t+1)=fymod;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The last thing to do in the loop is to compute phifilt ,the linear part of the
% gradient. This is obtained by introducing the two auxiliary varibles yfilt
% and ufilt.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
	if na>0 % Added handling of FIR case  
   	 thetad=thetaoe(t,1:na)';
 		 if t<nd
   	    yfilt(t+1)=-thetad'*[yfilt(t:-1:1);zeros(nd-t,1)]+ymod(t+1);
    		 ufilt(t+1)=-thetad'*[ufilt(t:-1:1);zeros(nd-t,1)]+u(t+1);
  	    else
    		 yfilt(t+1)=-thetad'*yfilt(t:-1:t-nd+1)+ymod(t+1);
    		 ufilt(t+1)=-thetad'*ufilt(t:-1:t-nd+1)+u(t+1);
       end;
   else % Added handling of FIR case
      yfilt(t+1)=ymod(t+1);
      ufilt(t+1)=u(t+1);
   end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% After creating yfilt and ufilt it is a simple matter to create the filtered
% regression vector at time t+1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  	if na>0 % Added handling of FIR case 		  
  	  	if t<na
     		phifilty=[yfilt(t:-1:1);zeros(na-t,1)];
  		else
   		phifilty=yfilt(t:-1:t-na+1);
  		end;
	   if t<nb
    		phifiltu=[ufilt(t:-1:1);zeros(nb-t,1)];
  		else
    		phifiltu=ufilt(t:-1:t-nb+1);
  		end;
        phifilt=[-1*phifilty;phifiltu];
   else % Added handling of FIR case
	   if t<nb
    		phifiltu=[ufilt(t:-1:1);zeros(nb-t,1)];
  		else
    		phifiltu=ufilt(t:-1:t-nb+1);
  		end;
        phifilt=[phifiltu];
   end        
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% To complete the gradient calculation phifilt is multiplied with df/dy and
% the total gradient psi is formed.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  phifilt=dfdy*phifilt;
  psi=[phifilt;psify0;psinon];
%
% That completes the loop
%
  t=t+1;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Variables that are not used by subsequent programs are cleared.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
clear ufilt;
clear yfilt;
clear ymod;
clear t;
clear lambda;
clear theta0;
clear fymod;
clear a;
clear b;
clear d;
clear nd;
clear phi;
clear deltaphi;
clear phifilt;
clear phifilty;
clear phifiltu;
clear theta0;
clear thetapro;
clear thetad;
clear S;
clear L;
clear zerovec;
clear delayu;
clear delayy;
clear delay;
clear thetalin0;
clear psi;
clear Intervals;
clear i;
clear ylow;
clear yhigh;
clear fylow;
clear fyhigh;
clear psinon;
clear psify0;
clear size_thetanon_0;
clear inside;
clear dfdy;
clear thetanon0;
clear size_P_0;
pack;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write parametervector.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
thetalinear=thetaoe(n-1,1:na+nb);
thetanonlinear=thetaoe(n-1,na+nb+1:na+nb+1+kmi+kpl);
thetalinear
thetanonlinear
