%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Developed by:			Torbjrn Wigren
%                       Division of Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       P O Box 337
%                       S-751 05 Uppsala
%                       SWEDEN
%
% Version:					1.0, PC MATLAB
%                       2.0, MATLAB 5
%
% Date:		        		rev. 1.0, Nov 10 1991
%                       rev. 2.0, March 28 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       DESCRIPTION
%
% This M-file implements a RPE-algorithm that identifies a static
% nonlinearity. The nonlinearity is approximated with a piecevise linear curve.
% The algorithm is referred to as algorithm 9 in Wigren 1990.
%
% Variables:         u = input signal
%                    yn = output signal
%                    n = number of samples
%                    algnr = number of algorithm (=9)
%                    fymod = output from the nonlinearity
%                    ynmod = output from the nonlinear model (=fymod)
%		               thetaoe = parameter vector containing f(yi)
%                    thetapro = dummy vector used in updating of thetaoe
%		               theta0 = initial value of parameter vector
%		               lambda = gain sequence
%		               perr = nonlinear prediction error
%		               S = scalar in Riccattieequation
%		               L = gain vector
%		               P = P-matrix in Riccattieequation
%		               lambda0 = rate of gain sequence (0.99 in L&S)
%		               kpl = kplus in the notes
%		               kmi = kminus in the notes
%                    k0 = slope used in initialization of the algorithm
%                    grid = vector containing grid points
%		               thetanon_0 = starting value of nonlinear parameter vector
%		               psi = total gradient vector
%		               Intervals = all interval where u < upper interval limit
%		               i = upper limit index in interval to which u belongs
%		               ulow = lower limit of interval to which u belongs
%                    uhigh = upper limit of interval to which u belongs
%                    fulow = parameter value in ulow
%                    fuhigh = parameter value in uhigh
%		               psinon = nonlinear part of the gradient
%		               psify0 = df/dfy0
%                    theta = estimated parameters
%                    lambda_0 = initial value of gain sequence
%                    P_0 = initial value of P-matrix
%                    size_P_0 = dimension of P_0
%                    thetanon0 = dummy variable (=thetanon_0)
%                    size_thetanon_0 = dimension of thetanon_0
%                    thetafy0_0 = bias parameter used in initialization
%                    inside = flag determining if inside all intervals
%
% The following variables must be present in the working are before the
% program is run (i.e. user initializations or data):
%
%                    n = number of data samples
%                    u = vector containing input signal samples
%                    yn = vector containing output signal samples
%                    k0 = slope used in initialization of algorithm
%                    thetafy0_0 = bias parameter used in initialization
%                    kmi = kminus in the notes
%                    kpl = kplus in the notes
%                    grid = vector of grid points
%                    lambda0 = rate of gain sequence
%                    lambda_0 = initial value of gain sequence
%                    P_0 = initial value of P_matrix
%                    thetanon_0 = initial value of nonlinearity (column vec)
%
% Note that by specifying P_0 =[], the default value of a unit matrix of
% appropriate dimension is applied. If thetanon_0 = [] is choosen, then a
% straight line initial value with slope k0 and bias thetafy_0 is used.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% User initializations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
lambda=lambda_0;
size_P_0=size(P_0);
if size_P_0(1,1)==0
  P_0=1*eye(1+kpl+kmi);
  P=P_0;
end;
if size_P_0(1,1)>0
  P=P_0;
end;
size_thetanon_0=size(thetanon_0);
if size_thetanon_0(1,1)==0
  thetanon_0=[k0*grid(1:kmi)';k0*grid((kmi+2):(kmi+kpl+1))']+thetafy0_0;
  thetanon0=thetanon_0;
end;
if size_thetanon_0(1,1)>0
  thetanon0=thetanon_0;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Other standard initializations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
algnr=9;
theta0=[k0*grid(kmi+2);thetanon0];
psi=zeros(1+kmi+kpl,1);
perr=zeros(n,1);
thetaoe=zeros(n,1+kmi+kpl);
ynmod=zeros(n,1);
fymod=0;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now the recursive algorithm starts. t is the time.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
inside=1;
t=1;
while t < n
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute prediction errors using prediction from previous iteration and
% save the result for later use.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  perr(t)=yn(t)-ynmod(t);
  if inside==0
    perr(t)=0;
  end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Then a new lambda-value is computed.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  lambda=lambda0*lambda+1-lambda0;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The next step is to iterate the Riccattieequation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  S=psi'*P*psi+lambda;
  P=(P-P*psi*psi'*P/S)/lambda;
  L=P*psi;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next the thetaoe-vector is uppdated and saved for later use. To get the same
% time indices for u, yn, ynmod, perr and thetaoe we distinguish between the
% cases t=1 and t>1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  if t==1
    thetapro=theta0+L*perr(t);
  end;
  if t>=2
    thetapro=thetaoe(t-1,:)'+L*perr(t);
  end;
  thetaoe(t,:)=thetapro';
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now the output of the nonlinearity is computed, together with
% the gradient. There are four cases to consider depending
% on which approximation interval u(t+1) belongs to.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  psify0=0;
  psinon=zeros(kmi+kpl,1);
  inside=0;
  if u(t+1) > grid(1);
    if u(t+1) < grid(kmi+kpl+1);
      inside=1;
      Intervals=find(grid > u(t+1));
      i=Intervals(1);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
      if i<=kmi
        ulow=grid(i-1);
        uhigh=grid(i);
        fulow=thetaoe(t,1+i-1);
        fuhigh=thetaoe(t,1+i);
        fymod=(fulow*uhigh-fuhigh*ulow+(fuhigh-fulow)*u(t+1))/(uhigh-ulow);
        psinon(i-1)=(uhigh-u(t+1))/(uhigh-ulow);
        psinon(i)=1-psinon(i-1);
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 2.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
     if i==kmi+1
        ulow=grid(i-1);
        uhigh=grid(i);
        fulow=thetaoe(t,1+i-1);
        fuhigh=thetaoe(t,1);
        fymod=(fulow*uhigh-fuhigh*ulow+(fuhigh-fulow)*u(t+1))/(uhigh-ulow);
        psinon(i-1)=(uhigh-u(t+1))/(uhigh-ulow);
        psify0=(u(t+1)-ulow)/(uhigh-ulow);
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 3.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
      if i==kmi+2
        ulow=grid(i-1);
        uhigh=grid(i);
        fulow=thetaoe(t,1);
        fuhigh=thetaoe(t,1+i-1);
        fymod=(fulow*uhigh-fuhigh*ulow+(fuhigh-fulow)*u(t+1))/(uhigh-ulow);
        psinon(i-1)=(u(t+1)-ulow)/(uhigh-ulow);
        psify0=(uhigh-u(t+1))/(uhigh-ulow);
      end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Case 4.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
      if i>kmi+2
        ulow=grid(i-1);
        uhigh=grid(i);
        fulow=thetaoe(t,1+i-2);
        fuhigh=thetaoe(t,1+i-1);
        fymod=(fulow*uhigh-fuhigh*ulow+(fuhigh-fulow)*u(t+1))/(uhigh-ulow);
        psinon(i-2)=(uhigh-u(t+1))/(uhigh-ulow);
        psinon(i-1)=1-psinon(i-2);
      end;
    end;
  end;
  ynmod(t+1)=fymod;
  psi=[psify0;psinon];
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% That completes the loop.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  t=t+1;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Variables that are not used by subsequent programs are cleared.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
clear fymod;
clear thetapro;
clear theta0;
clear lambda;
clear t;
clear S;
clear L;
clear psi;
clear Intervals;
clear i;
clear ulow;
clear uhigh;
clear fulow;
clear fuhigh;
clear psinon;
clear psify0;
clear size_P_0;
clear thetanon0;
clear size_thetanon_0;
clear inside;
pack;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write parameter vector.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
theta=thetaoe(n-1,:);
theta
