%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Developed by:			Torbjrn Wigren
%                       Division of Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       P O Box 337
%                       SE-751 05 Uppsala
%                       SWEDEN
%
% Version:					1.0, PC MATLAB
%                       2.0, MATLAB 5
%                       3.0, MATLAB 5
%
% Date:		        		rev. 1.0, Nov 10 1991
%                       rev. 2.0, March 28 2007
%                       rev. 3.0, April 10 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       DESCRIPTION
%
% This M-file creates a data vector consisting of input and nonlinear noisy
% outputs.
%
% First a discrete time time transfer function model (system description) 
% is simulated using a user chosen input. Then a nonlinear transformation 
% is applied and noise is added.
%
% The model is a model of neutralization titration of wastewater.
% The linear part is of first order and describes the linear mixing
% dynamics. The nonlinear part evaluates the severely nonlinear titration-
% curve.
%
% Variables:        ts = sampling time
%                   tu = basic clock period of input (times sampling time)
%                   amp = amplitude of input signal / uniform distribution
%                   n = length of data vector to be created
%                   u = input signal vector, in this case concentration of HCl
%                   bias = bias in input signal determining the operating point
%                   y = linear output vector
%                   fy = nonlinearly transformed linear output
%                   yn = nonlinear measurement signal with additive noise
%                   udummy = dummy variable
%                   lastrand = dummy variable
%                   i = loopcounter
%                   j = loopcounter
%                   bw = bandwith of first order continuous time noise model
%                   r1 = variance of continuous noise model
%                   d0dis = numerator of sampled noise model
%                   c0dis = denominator of sampled noise model
%                   e = white input noise to drive the noise filter
%                   w = noise to be added to nonlinear output
%                   H = concentration of hydrogen ions
%		              kw = ion product of water
%                   kchem = dissociation constant of NH3
%		              ka = dissosiation constant of weak acid
%                   MNaOh = concentration of sodium hydroxide before reaction
%		              MHCl = amount of hydrocloric acid used
%		              MNH3 = concentration of ammonia before reaction
%		              MHA = concentration of weak acid before reaction
%                   systtype = 0 if Wiener, 1 if static
%		              input_type = input signal type
%
% The following parameters must always be present in the working area:
%
%                   input_type = type of input signal
%                   u = vector of input samples if input_type=4
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting system parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
b0dis=[0.0 0.003626];
a0dis=[1 -0.8187];
systtype=0;
if length(a0dis)==1
    systtype=1;
end;
ts=20;
kw=1.0E-14;
kchem=5.6E-10;
ka=0.5E-5;
MNaOH=0.01;
MHA=0.005;
MNH3=0.01;
MHCl=(0.0:0.00006:0.03);
bias=0.750;
amp=0.730;
tu=1;
if input_type < 4
   tu=6;
   n=999;
end;
bw=1;
r1=0.1;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create pH-curve as a function of MHCl by solving x*x*x*x+a*x*x*x+b*x*x
% +c*x+d=0  using the Newton-Raphson method.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
pH=0*MHCl;
H=1E-14/MNaOH;
for i=1:length(MHCl)
  ak=MNH3-MHCl(i)+MNaOH+kchem+ka;
  bk=(kchem+ka)*MNaOH-(kchem+ka)*MHCl(i)-kw+MNH3*ka+kchem*ka-ka*MHA;
  ck=MNaOH*kchem*ka-kw*(ka+kchem)-MHCl(i)*kchem*ka-ka*kchem*MHA;
  dk=-kchem*ka*kw;
  for j=1:5
    H=abs(H-(H*H*H*H+ak*H*H*H+bk*H*H+ck*H+dk)/(4*H*H*H+3*ak*H*H+2*bk*H+ck));
  end;
  pH(i)=-1*log10(H);
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next an input is created. Several alternatives are available. Choose one
% by specifying the parameter input_type. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type==1
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Approximated PRBS.
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  u=amp*sign(randn(1,ceil(n/tu)));
  for i=1:tu-1
    u=[u;u(1:1, :)];
  end
  u=u(:);
end;
if input_type==2
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Uniformly distributed white noise input.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  u=2*amp*rand(1,ceil(n/tu));
  for i=1:tu-1
    u=[u;u(1:1, :)];
  end
  u=u(:)-amp;
end;
if input_type==3
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Approximated PRBS where each time interval of constant signal level is
% multiplied with a random factor with uniformly distributed amplitude.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  udummy=sign(randn(1,ceil(n/tu)));
  for i=1:tu-1
    udummy=[udummy;udummy(1:1, :)];
  end
  lastrand=rand(1);
  u=amp*lastrand*udummy(:,1);
  for i=2:ceil(n/tu)
    if udummy(1,i)-udummy(1, (i-1)) == 0
      u=[u;lastrand*amp*udummy(:,i)];
    else
      lastrand=rand(1);
      u=[u;lastrand*amp*udummy(:,i)];
    end;
  end,
end;
if input_type==4
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If input_type=4 a user specified input signal is already present in
% the vector u.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  n=length(u);
  tu=1;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Bias is added to the input signal to get the correct operating point,
% unless input_type=4. After generating the input, the output of the
% linear part of the system is generated using digital filtering.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type < 4
  u=u+bias;
end;
y=filter(b0dis,a0dis,u);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next a nonlinear transformation is applied to the output y. The 
% pH-curve results in the following calculation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
fy=zeros(n,1);
for t=1:n
  intervals1=find(MHCl>y(t));
  if length(intervals1)>0
    i=intervals1(1);
    fy(t)=pH(i);
  end;
  if length(intervals1)==0
    fy(t)=pH(length(MHCl));
  end;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The last thing to do is to add measurement noise to the nonlinear output.
% To do this we a first order continuous time (sampled) noise model is used,
% where the variance and bandwith are varied. The total energy in the noise is
% left constant when the bandwith is varied. First default values are set.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
d0dis=[1];
c0dis=[1;-exp(-bw*ts)];
e=r1*sqrt((1-exp(-2*bw*ts)))*randn(length(fy),1);
w=filter(d0dis,c0dis,e);
yn=fy+w;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Not used variables are removed from the workspace
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type==3
  clear udummy;
  clear lastrand;
end;
clear d0dis;
clear c0dis;
clear y;
clear fy;
clear e;
clear w;
clear ak;
clear bk;
clear ck;
clear dk;
clear intervals1;
clear i;
clear j;
