%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Developed by:			Torbjrn Wigren
%                       Division of Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       P O Box 337
%                       SE-751 05 Uppsala
%                       SWEDEN
%
% Version:					1.0, PC MATLAB
%                       2.0, MATLAB 5
%                       3.0, MATLAB 5
%
% Date:		        		rev. 1.0, Nov 10 1991
%                       rev. 2.0, March 28 2007
%                       rev. 3.0, April 10 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       DESCRIPTION
%
% This M-file creates a data vector consisting of input and nonlinear noisy
% outputs.
%
% First a discrete time time transfer function model (system description) 
% is simulated using a user chosen input. Then a nonlinear transformation 
% is applied and noise is added.
%
% The model is a theoretical model that is piecevise quadratic in order to
% make a validation of the algorithm VIII possible.
%
% Variables:        ts = sampling time
%                   b0dis = discrete time system numerator
%                   a0dis = discrete time system denominator
%                   tu = basic clock period of input (times sampling time)
%                   amp = amplitude of input signal / uniform distribution
%                   n = length of data vector to be created
%                   u = input signal vector
%                   y = linear output vector
%                   fy = nonlinearly transformed linear output
%                   yn = nonlinear measurement signal with additive noise
%                   udummy = dummy variable
%                   lastrand = dummy variable
%                   i = loopcounter
%                   j = loopcounter
%                   bw = bandwith of first order continuous time noise model
%                   r1 = variance of continuous noise model
%                   d0dis = numerator of sampled noise model
%                   c0dis = denominator of sampled noise model
%                   e = white input noise to drive the noise filter
%                   w = noise to be added to nonlinear output
%		              bias = bias in input signal
%                   gridsys = gridpoints of system
%                   fygrid = function values of nonlinearity in gridpoints
%                   lowlimit = lower plotting limit
%                   highlimit = higher plotting limit
%                   delta = plotting spacing
%                   dummy = auxilary variable
%                   ylow = lower limit of current interval
%                   ymid = mid point of current interval
%                   yhigh = higher limit of current interval
%                   fylow = function value in ylow
%                   fymid = function value in ymid
%                   fyhigh = function value in yhigh
%                   systtype = 0 if Wiener, 1 if static
%                   input_type = type of input signal
%
% The following parameters must always be present in the working area:
%
%                   input_type = type of input signal
%                   u = vector of input signal samples if input_type=4
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting system parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
b0dis=[0 0.0979 0.0830];
a0dis=[1 -1.3917 0.5703];
systtype=0;
if length(a0dis)==1
   systtype=1;
end;
ts=0.08;
gridsys=[-1.2 -0.75 -0.3 0.0 0.3 0.75 1.2];
fygrid=[0.2050 0.35125 0.7 1 1.3 1.64875 1.7950];
amp=0.9;
bias=0;
tu=1;
if input_type < 4
   tu=5;
   n=999;
end;
bw=20;
r1=0.001;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next an input is created. Several alternatives are available. Choose one
% by specifying the variable input_type. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type==1
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Approximated PRBS.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  u=amp*sign(randn(1,ceil(n/tu)));
  for i=1:tu-1
    u=[u;u(1:1, :)];
  end
  u=u(:);
end;
if input_type==2
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Uniformly distributed white noise input.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  u=2*amp*rand(1,ceil(n/tu));
  for i=1:tu-1
    u=[u;u(1:1, :)];
  end
  u=u(:)-amp;
end
if input_type==3
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Approximated PRBS where each time interval of constant signal level
% is multiplied with a random factor that is uniformly distribted.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  udummy=sign(randn(1,ceil(n/tu)));
  for i=1:tu-1
    udummy=[udummy;udummy(1:1, :)];
  end
  lastrand=rand(1);
  u=amp*lastrand*udummy(:,1);
  for i=2:ceil(n/tu)
    if udummy(1,i)-udummy(1, (i-1)) == 0
      u=[u;lastrand*amp*udummy(:,i)];
    else
      lastrand=rand(1);
      u=[u;lastrand*amp*udummy(:,i)];
    end  
  end
end
if input_type==4
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In this case the input signal is already present in the vector u.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  n=length(u);
  tu=1;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% After generating the input the output of the linear part of the system
% is generated using digital filtering. First bias is added to the input 
% signal though.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type < 4
  u=u+bias;
end;
y=filter(b0dis,a0dis,u);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next a nonlinear transformation is applied to the output y
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
fy=y;
for i=1:n
  for j=1:2:length(gridsys)-2
    if y(i)>gridsys(j)
      if y(i)<=gridsys(j+2)
        ylow=gridsys(j);
        ymid=gridsys(j+1);
        yhigh=gridsys(j+2);
        fylow=fygrid(j);
        fymid=fygrid(j+1);
        fyhigh=fygrid(j+2);
        dummy=2*(fyhigh-2*fymid+fylow)*(y(i)-ymid)*(y(i)-ymid);
        fy(i)=fymid+(fyhigh-fylow)/(yhigh-ylow)*(y(i)-ymid);
        fy(i)=fy(i)+dummy/((yhigh-ylow)*(yhigh-ylow));
      end;
    end;
  end;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The last thing to do is to add measurement noise to the nonlinear output.
% To do this a first order continuous time (sampled) noise model is used where
% the variance and bandwith are varied. The total energy in the noise is left
% constant when the bandwith is varied.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
d0dis=[1];
c0dis=[1;-exp(-bw*ts)];
e=r1*sqrt((1-exp(-2*bw*ts)))*randn(length(fy),1);
w=filter(d0dis,c0dis,e);
yn=fy+w;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% variables not used by subsequent programs are cleared.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type==3
  clear udummy;
  clear lastrand;
end;
clear c0dis;
clear d0dis;
clear y;
clear fy;
clear e;
clear w;
clear dummy;
clear lowlimit;
clear highlimit;
clear delta;
clear ylow;
clear xlow;
clear ymid;
clear xmid;
clear yhigh;
clear xhigh;
clear fylow;
clear fxlow;
clear fymid;
clear fxmid;
clear fyhigh;
clear fxhigh;
clear i;
clear j;
