%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Developed by:			Torbjrn Wigren
%                       Division of Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       P O Box 27
%                       S-751 05 Uppsala
%                       SWEDEN
%
% Version:					1.0, PC MATLAB
%                       2.0, MATLAB 5
%                       3.0, MATLAB 5
%
% Date:		        		rev. 1.0, Nov 10 1991
%                       rev. 2.0, March 28 2007
%                       rev. 3.0, April 10 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       DESCRIPTION
%
% This M-file creates a data vector consisting of input and nonlinear noisy
% outputs.
%
% First a discrete time time transfer function model (system description) 
% is simulated using a user chosen input. Then a nonlinear transformation 
% is applied and noise is added.
%
% The model describes a control-valve with a nonlinear stem-height to fluid
% characteristic. The dynamics from actuator to stem position is assumed
% to be described by a linear dynamic model.
%
% Variables:        ts = sampling time
%                   b0dis = discrete time system numerator
%                   a0dis = discrete time system denominator
%                   tu = basic clock period of input (times sampling time)
%                   amp = amplitude of input signal / uniform distribution
%                   n = length of data vector to be created
%                   u = input signal vector
%                   y = linear output vector
%                   fy = nonlinearly transformed linear output
%                   yn = nonlinear measurement signal with additive noise
%                   udummy = dummy variable
%                   lastrand = dummy variable
%                   i = loopcounter
%                   bw = bandwith of first order continuous time noise model
%                   r1 = variance of continuous noise model
%                   d0dis = numerator of sampled noise model
%                   c0dis = denominator of sampled noise model
%                   e = white input noise to drive the noise filter
%                   w = noise to be added to nonlinear output
%		    			  qm = maximum flow
%		              ym = maximum stem position
%                   alfa = installation dependent parameter
%                   systtype = 0 if Wiener, 1 if static
%                   input_type = type of input signal
%
% The following parameters must always be present in the working area:
%
%                   input_type = type of input signal
%                   u = vector of input samples if input_type=4 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%		    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting system parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
b0dis=[0 0.0616 0.0543];
a0dis=[1 -1.5714 0.6873];
systtype=0;
if length(a0dis)==1
   systtype=1;
end;
ts=0.075;
qm=1;
ym=1;
alfa=0.10;
bias=0.4;
amp=0.35;
tu=1;
if input_type<4
   n=999;
   tu=7;
end;
bw=20;
r1=0.01;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next an input is created. Several alternatives are available. Choose one
% by specifying the variable input_type. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type==1
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Approximated PRBS.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  u=amp*sign(randn(1,ceil(n/tu)));
  for i=1:tu-1
    u=[u;u(1:1, :)];
  end
  u=u(:);
end;
if input_type==2
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Uniformly distributed white noise input.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  u=2*amp*rand(1,ceil(n/tu));
  for i=1:tu-1
    u=[u;u(1:1, :)];
  end
  u=u(:)-amp;
end;
if input_type==3
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Approximated PRBS where each time interval of constant signal level
% is scaled with a uniformly distributed amplitude.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  udummy=sign(randn(1,ceil(n/tu)));
  for i=1:tu-1
    udummy=[udummy;udummy(1:1, :)];
  end
  lastrand=rand(1);
  u=amp*lastrand*udummy(:,1);
  for i=2:ceil(n/tu)
    if udummy(1,i)-udummy(1, (i-1)) == 0
      u=[u;lastrand*amp*udummy(:,i)];
    else
      lastrand=rand(1);
      u=[u;lastrand*amp*udummy(:,i)];
    end
  end
end;
if input_type==4
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If input_type=4 a userspecified input is already present in the variable u.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  n=length(u);
  tu=1;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% After generating the input the output of the linear part of the system
% is generated using digital filtering. First bias is added to the input 
% signal to give a correct operating point unless input_type=4.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type<4
  u=u+bias;
end;
y=filter(b0dis,a0dis,u);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next a nonlinear transformation is applied to the output y.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
fy=qm*(y/ym)./sqrt(alfa+(1-alfa)*(y/ym).*(y/ym));
for i=1:n
  if fy(i)>qm
    fy(i)=qm;
  end;
  if fy(i)<0
    fy(i)=0;
  end;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The last thing to do is to add measurement noise to the nonlinear output.
% To do this we use a first order continuous time (sampled) noise model where
% the variance and bandwith can be varied. The total energy in the noise is left
% constant when the bandwidth is varied.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
d0dis=[1];
c0dis=[1;-exp(-bw*ts)];
e=r1*sqrt((1-exp(-2*bw*ts)))*randn(length(fy),1);
w=filter(d0dis,c0dis,e);
yn=fy+w;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Delete variables that are not needed by other programs.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type==3
  clear udummy;
  clear lastrand;
end;
clear d0dis;
clear c0dis;
clear y;
clear fy;
clear e
clear w;
clear i;
