%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Developed by:			Torbjrn Wigren
%                       Division of Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       P O Box 337
%                       SE-751 05 Uppsala
%                       SWEDEN
%
% Version:					1.0, PC MATLAB
%                       2.0, MATLAB 5
%                       3.0, MATLAB 5
%
% Date:		        		rev. 1.0, Nov 10 1991
%                       rev. 2.0, March 28 2007
%                       rev. 3.0, April 10 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       DESCRIPTION
%
% This M-file validates the estimated Wiener model by plotting the output
% signal from the system, and the estimated model using the final value of 
% the parameter vector. The residual signal is plotted as well.
%
% Variables:		      aest = estimated a-polynomial
%                       best = estimated b-polynomial
%                       na = order of aest (must exist)
%                       nb = order of best (must exist)
%                       algnr = number of algorithm used (must exist)
%                       thetaoe = estimated parameter vector (must exist)
%                       n = number of data samples
%                       u = input signal
%                       yn = output signal of system
%                       ymod_val = output of linear part of the model
%                       ynmod_val = output of model
%                       fymod_val = static nonlinearity in ymod 
%                       fumod_val = static nonlinearity in u
%                       k0 = known slope of static nonlinearity (must exist)
%                       kmi = k-minus (must exist)
%                       kpl = k-plus (must exist)
%                       grid = vector of grid points (must exist)
%                       i = loop variable
%                       j = loop variable
%                       ylow = lower limit of current interval
%                       yhigh = higher limit of current interval
%                       fylow = function value of nonlinearity in ylow
%                       fyhigh = function value of nonlinearity in yhigh
%                       ulow = lower limit of current interval
%                       uhigh = higher limit of current interval
%                       fulow = function value of nonlinearity in ulow
%                       fuhigh = function value of nonlinearity in uhigh
%                       extgrid = extended grid (must exist)
%                       extgrid1 = copy of extended grid
%                       dummy = dummy variable
%                       ymid = mid value of current interval
%                       umid = mid value of current interval
%                       fymid = function value of nonlinearity in ymid
%                       fumid = function value of nonlinearity in umid
%                       residual = residual signal
%                       out_signal = plotting vector
%                       inside = flag determining if inside all intervals
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get aest and best from theta if not algorithm 9 or 10.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if algnr<9
  aest=[1 thetaoe(n-1,1:na)];
  best=[0 thetaoe(n-1,na+1:na+nb)];
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize some other variables.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
ymod_val=0*yn;
fymod_val=0*yn;
fumod_val=0*u;
ynmod_val=0*yn;
residual=0*yn;
inside=0*yn;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate output from linear part of model by digital filtering.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
ymod_val=filter(best,aest,u);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This signal must now be transformed with the static nonlinearity.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A priori known nonlinearity in case of algnr < 6.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if algnr < 6
  for i=1:n
    [fymod_val(i),not_used]=eval_fn(ymod_val(i));
    inside(i)=1;
  end;
ynmod_val=fymod_val;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Algorithm 7.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if algnr==7
  theta=[thetaoe(n-1,1:na+nb+1+kmi),k0*grid(kmi+1)+thetaoe(n-1,na+nb+1)];
  theta=[theta,k0*grid(kmi+2)+thetaoe(n-1,na+nb+1)];
  theta=[theta,thetaoe(n-1,na+nb+2+kmi:na+nb+1+kmi+kpl)];
  for i=1:n
    for j=1:kpl+kmi+1
      if ymod_val(i)>grid(j)
       if ymod_val(i)<grid(j+1)
        inside(i)=1;
        if abs(j-kmi-1)>0.1
         ylow=grid(j);
         yhigh=grid(j+1);
         fylow=theta(na+nb+1+j);
         fyhigh=theta(na+nb+2+j);
         fymod_val(i)=fylow*yhigh-fyhigh*ylow+fyhigh*ymod_val(i);
         fymod_val(i)=(fymod_val(i)-fylow*ymod_val(i))/(yhigh-ylow);
        end;
        if abs(j-kmi-1)<0.1
          fymod_val(i)=k0*ymod_val(i)+thetaoe(n-1,na+nb+1);
        end;
       end;
      end;
    end;
  end;
ynmod_val=fymod_val;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Algorithm 8.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if algnr==8
extgrid1=extgrid;
theta=[thetaoe(n-1,1:na+nb+1+2*kmi),k0*extgrid1(2*kmi+1)+thetaoe(n-1,na+nb+1)];
theta=[theta,0,k0*extgrid1(2*kmi+3)+thetaoe(n-1,na+nb+1)];
theta=[theta,thetaoe(n-1,na+nb+2*kmi+2:na+nb+2*kmi+2*kpl+1)];
for i=1:n
  for j=1:2:2*kpl+2*kmi+1
    if ymod_val(i)>extgrid1(j)
      if ymod_val(i)<extgrid1(j+2)
        inside(i)=1;
        if abs(j-2*kmi-1)>0.1
          ylow=extgrid1(j);
          ymid=extgrid1(j+1);
          yhigh=extgrid1(j+2);
          fylow=theta(na+nb+1+j);
          fymid=theta(na+nb+2+j);
          fyhigh=theta(na+nb+3+j);
          dummy=2*(fyhigh-2*fymid+fylow)*(ymod_val(i)-ymid)*(ymod_val(i)-ymid);
          fymod_val(i)=fymid+(fyhigh-fylow)/(yhigh-ylow)*(ymod_val(i)-ymid);
          fymod_val(i)=fymod_val(i)+dummy/((yhigh-ylow)*(yhigh-ylow));
          dummy=4*(fyhigh-2*fymid+fylow)*(ymod_val(i)-ymid)/(yhigh-ylow);
        end;
        if abs(j-2*kmi-1)<0.1
          fymod_val(i)=k0*ymod_val(i)+thetaoe(n-1,na+nb+1);
        end;
       end;
      end;
    end;
  end;
ynmod_val=fymod_val;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Algorithm 9.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if algnr==9
  theta=[thetaoe(n-1,2:1+kmi),thetaoe(n-1,1)];
  theta=[theta,thetaoe(n-1,2+kmi:1+kmi+kpl)];
  for i=1:n
    for j=1:kpl+kmi+1
      if u(i)>grid(j)
       if u(i)<grid(j+1)
         inside(i)=1;
         ulow=grid(j);
         uhigh=grid(j+1);
         fulow=theta(j);
         fuhigh=theta(1+j);
         fumod_val(i)=fulow*uhigh-fuhigh*ulow+fuhigh*u(i)-fulow*u(i);
         fumod_val(i)=fumod_val(i)/(uhigh-ulow);
       end;
      end;
    end;
  end;
ynmod_val=fumod_val;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Algorithm 10.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if algnr==10
extgrid1=extgrid;
theta=[thetaoe(n-1,2:1+2*kmi),thetaoe(n-1,1)];
theta=[theta,thetaoe(n-1,2*kmi+2:2*kmi+2*kpl+1)];
for i=1:n
  for j=1:2:2*kpl+2*kmi+1
    if u(i)>extgrid1(j)
      if u(i)<extgrid1(j+2)
        inside(i)=1;
        ulow=extgrid1(j);
        umid=extgrid1(j+1);
        uhigh=extgrid1(j+2);
        fulow=theta(j);
        fumid=theta(1+j);
        fuhigh=theta(2+j);
        dummy=2*(fuhigh-2*fumid+fulow)*(u(i)-umid)*(u(i)-umid);
        fumod_val(i)=fumid+(fuhigh-fulow)/(uhigh-ulow)*(u(i)-umid);
        fumod_val(i)=fumod_val(i)+dummy/((uhigh-ulow)*(uhigh-ulow));
        dummy=4*(fuhigh-2*fumid+fulow)*(u(i)-umid)/(uhigh-ulow);
       end;
      end;
    end;
  end;
ynmod_val=fumod_val;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Take away zeros by filling in latest output value instead to faciliate 
% plotting.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
for i=2:n
  if ynmod_val(i)==0
    ynmod_val(i)=ynmod_val(i-1);
  end;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate residual signal.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
residual=yn-ynmod_val;
residual=residual.*inside;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate total vector containing output of system and model.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
out_signal=[yn';ynmod_val']';
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Start plotting.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
subplot(211)
plot(out_signal(1:n,:))
xlabel('time  [samples]');
ylabel('system/model output');
title('(a)');
subplot(212)
plot(residual(1:n))
xlabel('time  [samples]');
ylabel('residual');
title('(b)');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Variables that are not used by subsequent programs are cleared.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if algnr < 9
  clear aest;
  clear best;
end;
clear ymod_val;
clear ynmod_val;
clear fymod_val;
clear i;
clear residual;
clear out_signal;
if algnr > 5
  clear inside;
  if algnr < 9
    clear j;
    clear ylow;
    clear yhigh;
    clear fylow;
    clear fyhigh;
    if algnr==8
      clear extgrid1;
      clear dummy;
      clear ymid;
      clear fymid;
    end;
  end;
end;
if algnr > 8
  clear fumod_val;
  clear j;
  clear ulow;
  clear uhigh;
  clear fulow;
  clear fuhigh;
  if algnr==10;
    clear extgrid1;
    clear dummy;
    clear umid;
    clear fumid;
  end;
end;
if algnr < 6
  clear not_used;
end;

